@@ -1,9 +1,8 @@
 use matrixmultiply::{dgemm, sgemm};
-use tensorgraph_sys::device::cpu::Cpu;
 
 use crate::blas::{MatrixOp, GEMM};
 
-impl GEMM<(), Cpu> for f32 {
+impl GEMM<()> for f32 {
     unsafe fn gemm(
         _ctx: (),
         transa: MatrixOp,
@@ -44,7 +43,7 @@ impl GEMM<(), Cpu> for f32 {
     }
 }
 
-impl GEMM<(), Cpu> for f64 {
+impl GEMM<()> for f64 {
     unsafe fn gemm(
         _ctx: (),
         transa: MatrixOp,
