@@ -1,19 +1,33 @@
 use std::mem::MaybeUninit;
 
-use num_traits::{One, Zero};
-
 use tensorgraph_sys::{
-    device::{DefaultDeviceAllocator, Device, DeviceAllocator},
     ptr::Ref,
-    View, ViewMut, {DefaultVec, Vec},
+    View, ViewMut,
 };
 
 use crate::{
-    blas::{BLASContext, DefaultBLASContext, MatrixOp, GEMM},
     dims::{Dimension, RemoveDim},
     storage::{IntoOwned, Storage, StorageMut},
 };
 
+mod matrix;
+pub use matrix::*;
+
+/// A representation of a slice
+pub type Slice<T, D> = Ref<[T], D>;
+
+/// Gets the view repr of the provided storage
+pub type ViewOf<S> = Slice<<S as Storage>::T, <S as Storage>::Device>;
+
+/// A 'view' of a tensor, Like `&[T]` is to `Vec<T>`
+pub type TensorView<'a, T, D, Dim> = Tensor<&'a Slice<T, D>, Dim>;
+
+/// A 'mut view' of a tensor, Like `&mut [T]` is to `Vec<T>`
+pub type TensorViewMut<'a, T, D, Dim> = Tensor<&'a mut Slice<T, D>, Dim>;
+
+/// An uninit tensor. Contents are mutable and specified as [`MaybeUninit`].
+pub type UninitTensor<'a, T, D, Dim> = TensorViewMut<'a, MaybeUninit<T>, D, Dim>;
+
 /// A multidimensional data structure not unlike [`ndarray::ArrayBase`](https://docs.rs/ndarray/0.15.4/ndarray/struct.ArrayBase.html).
 pub struct Tensor<S: Storage, Dim: Dimension> {
     shape: Dim,
@@ -122,207 +136,6 @@ impl<S: Storage, Dim: Dimension> Tensor<S, Dim> {
     }
 }
 
-/// A representation of a slice
-pub type Slice<T, D> = Ref<[T], D>;
-
-/// Gets the view repr of the provided storage
-pub type ViewOf<S> = Slice<<S as Storage>::T, <S as Storage>::Device>;
-
-/// A 'view' of a tensor, Like `&[T]` is to `Vec<T>`
-pub type TensorView<'a, T, D, Dim> = Tensor<&'a Slice<T, D>, Dim>;
-
-/// A 'mut view' of a tensor, Like `&mut [T]` is to `Vec<T>`
-pub type TensorViewMut<'a, T, D, Dim> = Tensor<&'a mut Slice<T, D>, Dim>;
-
-/// An uninit tensor. Contents are mutable and specified as [`MaybeUninit`].
-pub type UninitTensor<'a, T, D, Dim> = TensorViewMut<'a, MaybeUninit<T>, D, Dim>;
-
-/// A 2-dimensional tensor
-pub type Matrix<S> = Tensor<S, [usize; 2]>;
-
-/// A 'view' of a matrix, Like `&[T]` is to `Vec<T>`
-pub type MatrixView<'a, T, D> = Matrix<&'a Slice<T, D>>;
-
-/// A 'mut view' of a matrix, Like `&mut [T]` is to `Vec<T>`
-pub type MatrixViewMut<'a, T, D> = Matrix<&'a mut Slice<T, D>>;
-
-/// An uninit matrix. Contents are mutable and specified as [`MaybeUninit`].
-pub type UninitMatrix<'a, T, D> = MatrixViewMut<'a, MaybeUninit<T>, D>;
-
-impl<S: Storage> Matrix<S> {
-    /// Multiply two matricies together.
-    pub fn dot(&self, rhs: Matrix<&ViewOf<S>>) -> Matrix<DefaultVec<S::T, S::Device>>
-    where
-        S::T: Zero + One,
-        S::Device: DefaultDeviceAllocator + DefaultBLASContext,
-        S::T: GEMM<<S::Device as DefaultBLASContext>::Context, S::Device>,
-    {
-        self.dot_using(rhs, Default::default())
-    }
-
-    /// Multiply two matricies together, using the specified [`BLASContext`]
-    pub fn dot_using<C: BLASContext<S::Device>>(
-        &self,
-        rhs: Matrix<&ViewOf<S>>,
-        ctx: C,
-    ) -> Matrix<DefaultVec<S::T, S::Device>>
-    where
-        S::T: Zero + One,
-        S::Device: DefaultDeviceAllocator,
-        S::T: GEMM<C, S::Device>,
-    {
-        self.dot_into(rhs, ctx, Default::default())
-    }
-
-    /// Multiply two matricies together, using the provided [`DeviceAllocator`], using the specified [`BLASContext`]
-    pub fn dot_into<C: BLASContext<S::Device>, A: DeviceAllocator<S::Device>>(
-        &self,
-        rhs: Matrix<&ViewOf<S>>,
-        ctx: C,
-        alloc: A,
-    ) -> Matrix<Vec<S::T, A, S::Device>>
-    where
-        S::T: Zero + One,
-        S::T: GEMM<C, S::Device>,
-    {
-        let rows = self.shape[0];
-        let cols = rhs.shape[1];
-        let mut v = Vec::with_capacity_in(rows * cols, alloc);
-        unsafe {
-            let uninit =
-                Matrix::from_shape([rows, cols], &mut v.space_capacity_mut()[..rows * cols]);
-
-            gemm_uninit_ctx(ctx, S::T::one(), self.view(), rhs, uninit);
-
-            v.set_len(rows * cols);
-        }
-        Matrix::from_shape([rows, cols], v)
-    }
-}
-
-impl<'a, T, D: Device, Dim: Dimension> UninitTensor<'a, T, D, Dim> {
-    /// # Safety
-    /// Contents must be initialised
-    pub unsafe fn assume_init(self) -> TensorViewMut<'a, T, D, Dim> {
-        Tensor {
-            shape: self.shape,
-            strides: self.strides,
-            data: self.data.assume_init_mut(),
-        }
-    }
-}
-
-impl<'a, T, D: Device, Dim: Dimension> TensorView<'a, MaybeUninit<T>, D, Dim> {
-    /// # Safety
-    /// Contents must be initialised
-    pub unsafe fn assume_init(self) -> TensorView<'a, T, D, Dim> {
-        Tensor {
-            shape: self.shape,
-            strides: self.strides,
-            data: self.data.assume_init(),
-        }
-    }
-}
-
-/// Performs the basic matmul operation.
-/// C = alpha * A * B.
-///
-/// Uses the default [`BLASContext`] for the device.
-pub fn gemm_uninit<F: GEMM<D::Context, D> + Zero, D: DefaultBLASContext>(
-    alpha: F,
-    a: Matrix<impl Storage<T = F, Device = D>>,
-    b: Matrix<impl Storage<T = F, Device = D>>,
-    c: UninitMatrix<F, D>,
-) {
-    gemm_uninit_ctx(D::Context::default(), alpha, a, b, c)
-}
-
-/// Performs the basic matmul operation.
-/// C = alpha * A * B.
-pub fn gemm_uninit_ctx<F: GEMM<C, D> + Zero, C: BLASContext<D>, D: Device>(
-    ctx: C,
-    alpha: F,
-    a: Matrix<impl Storage<T = F, Device = D>>,
-    b: Matrix<impl Storage<T = F, Device = D>>,
-    c: UninitMatrix<F, D>,
-) {
-    // Safety:
-    // Specifying beta == 0.0 should allow c to be safely read while uninitialised
-    unsafe { gemm_ctx(ctx, alpha, a, b, F::zero(), c.assume_init()) }
-}
-
-/// Performs the basic matmul operation.
-/// C = alpha * A * B + beta * C.
-///
-/// Uses the default [`BLASContext`] for the device.
-pub fn gemm<F: GEMM<D::Context, D> + Zero, D: DefaultBLASContext>(
-    alpha: F,
-    a: Matrix<impl Storage<T = F, Device = D>>,
-    b: Matrix<impl Storage<T = F, Device = D>>,
-    beta: F,
-    c: MatrixViewMut<F, D>,
-) {
-    gemm_ctx(D::Context::default(), alpha, a, b, beta, c)
-}
-
-/// Performs the basic matmul operation.
-/// C = alpha * A * B + beta * C.
-pub fn gemm_ctx<F: GEMM<C, D> + Zero, C: BLASContext<D>, D: Device>(
-    ctx: C,
-    alpha: F,
-    a: Matrix<impl Storage<T = F, Device = D>>,
-    b: Matrix<impl Storage<T = F, Device = D>>,
-    beta: F,
-    c: MatrixViewMut<F, D>,
-) {
-    let [rowsa, colsa] = a.shape;
-    let [rowsb, colsb] = b.shape;
-    let [rowsc, colsc] = c.shape;
-    assert_eq!(rowsa, rowsc);
-    assert_eq!(colsb, colsc);
-    assert_eq!(colsa, rowsb);
-
-    let m = rowsa as i32;
-    let k = rowsb as i32;
-    let n = colsb as i32;
-
-    let (transa, lda) = lead(a.strides);
-    let (transb, ldb) = lead(b.strides);
-
-    // C must not be transposed
-    assert_eq!(c.strides[0], 1);
-    let ldc = c.strides[1] as i32;
-
-    unsafe {
-        F::gemm(
-            ctx,
-            transa,
-            transb,
-            m,
-            n,
-            k,
-            alpha,
-            a.data.as_ref().as_ptr(),
-            lda,
-            b.data.as_ref().as_ptr(),
-            ldb,
-            beta,
-            c.data.as_ptr(),
-            ldc,
-        )
-    }
-}
-
-fn lead(s: [usize; 2]) -> (MatrixOp, i32) {
-    if s[0] == 1 {
-        (MatrixOp::NoTrans, s[1] as i32)
-    } else if s[1] == 1 {
-        (MatrixOp::Trans, s[0] as i32)
-    } else {
-        panic!("one of the strides must be 1 (contiguous)")
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use std::ops::Deref;
@@ -388,6 +201,9 @@ mod tests {
 
     #[test]
     fn slice() {
+        //     0 2 4
+        // A = 1 3 5
+
         // column major
         let a = [0., 1., 2., 3., 4., 5.];
         let a = Tensor::from_shape([2, 3], a);
